@@ -74,6 +74,8 @@
  *
  */
 
+#include <errno.h>
+#include <string.h>
 				/* Driver data structures */
 #include "radeon.h"
 #include "radeon_reg.h"
@@ -131,7 +133,7 @@ void RADEONWaitForFifoFunction(ScrnInfoPtr pScrn, int entries)
 		INREG(RADEON_RBBM_STATUS) & RADEON_RBBM_FIFOCNT_MASK;
 	    if (info->fifo_slots >= entries) return;
 	}
-	RADEONTRACE(("FIFO timed out: %d entries, stat=0x%08x\n",
+	RADEONTRACE(("FIFO timed out: %u entries, stat=0x%08x\n",
 		     INREG(RADEON_RBBM_STATUS) & RADEON_RBBM_FIFOCNT_MASK,
 		     INREG(RADEON_RBBM_STATUS)));
 	xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
@@ -154,16 +156,16 @@ void RADEONEngineFlush(ScrnInfoPtr pScrn)
     unsigned char *RADEONMMIO = info->MMIO;
     int            i;
 
-    OUTREGP(RADEON_RB2D_DSTCACHE_CTLSTAT,
-	    RADEON_RB2D_DC_FLUSH_ALL,
-	    ~RADEON_RB2D_DC_FLUSH_ALL);
+    OUTREGP(RADEON_RB3D_DSTCACHE_CTLSTAT,
+	    RADEON_RB3D_DC_FLUSH_ALL,
+	    ~RADEON_RB3D_DC_FLUSH_ALL);
     for (i = 0; i < RADEON_TIMEOUT; i++) {
-	if (!(INREG(RADEON_RB2D_DSTCACHE_CTLSTAT) & RADEON_RB2D_DC_BUSY))
+	if (!(INREG(RADEON_RB3D_DSTCACHE_CTLSTAT) & RADEON_RB3D_DC_BUSY))
 	    break;
     }
     if (i == RADEON_TIMEOUT) {
 	RADEONTRACE(("DC flush timeout: %x\n",
-		    INREG(RADEON_RB2D_DSTCACHE_CTLSTAT)));
+		    INREG(RADEON_RB3D_DSTCACHE_CTLSTAT)));
     }
 }
 
@@ -254,8 +256,8 @@ void RADEONEngineReset(ScrnInfoPtr pScrn)
 					RADEON_SOFT_RESET_E2));
 	INREG(RADEON_RBBM_SOFT_RESET);
 	OUTREG(RADEON_RBBM_SOFT_RESET, 0);
-	tmp = INREG(RADEON_RB2D_DSTCACHE_MODE);
-	OUTREG(RADEON_RB2D_DSTCACHE_MODE, tmp | (1 << 17)); /* FIXME */
+	tmp = INREG(RADEON_RB3D_DSTCACHE_MODE);
+	OUTREG(RADEON_RB3D_DSTCACHE_MODE, tmp | (1 << 17)); /* FIXME */
     } else {
 	OUTREG(RADEON_RBBM_SOFT_RESET, (rbbm_soft_reset |
 					RADEON_SOFT_RESET_CP |
@@ -602,6 +604,22 @@ void RADEONCPReleaseIndirect(ScrnInfoPtr pScrn)
 			&indirect, sizeof(drmRadeonIndirect));
 }
 
+/** \brief Calculate HostDataBlit parameters from pointer and pitch
+ *
+ * This is a helper for the trivial HostDataBlit users that don't need to worry
+ * about tiling etc.
+ */
+void
+RADEONHostDataParams(ScrnInfoPtr pScrn, CARD8 *dst, CARD32 pitch, int cpp,
+		     CARD32 *dstPitchOff, int *x, int *y)
+{
+    RADEONInfoPtr info = RADEONPTR( pScrn );
+    CARD32 dstOffs = dst - info->FB + info->fbLocation;
+
+    *dstPitchOff = pitch << 16 | (dstOffs & ~RADEON_BUFFER_ALIGN) >> 10;
+    *y = ( dstOffs & RADEON_BUFFER_ALIGN ) / pitch;
+    *x = ( ( dstOffs & RADEON_BUFFER_ALIGN ) - ( *y * pitch ) ) / cpp;
+}
 
 /* Set up a hostdata blit to transfer data from system memory to the
  * framebuffer. Returns the address where the data can be written to and sets
@@ -610,16 +628,17 @@ void RADEONCPReleaseIndirect(ScrnInfoPtr pScrn)
 CARD8*
 RADEONHostDataBlit(
     ScrnInfoPtr pScrn,
-    unsigned int bpp,
+    unsigned int cpp,
     unsigned int w,
-    CARD32 dstPitch,
+    CARD32 dstPitchOff,
     CARD32 *bufPitch,
-    CARD8* *dst,
+    int x,
+    int *y,
     unsigned int *h,
     unsigned int *hpass
 ){
     RADEONInfoPtr info = RADEONPTR( pScrn );
-    CARD32 format, dst_offs, dwords, x, y;
+    CARD32 format, dwords;
     CARD8 *ret;
     RING_LOCALS;
 
@@ -628,7 +647,7 @@ RADEONHostDataBlit(
 	return NULL;
     }
 
-    switch ( bpp )
+    switch ( cpp )
     {
     case 4:
 	format = RADEON_GMC_DST_32BPP;
@@ -636,17 +655,15 @@ RADEONHostDataBlit(
 	break;
     case 2:
 	format = RADEON_GMC_DST_16BPP;
-	w = (w + 1) & ~1;
-	*bufPitch = 2 * w;
+	*bufPitch = 2 * ((w + 1) & ~1);
 	break;
     case 1:
 	format = RADEON_GMC_DST_8BPP_CI;
-	w = (w + 3) & ~3;
-	*bufPitch = w;
+	*bufPitch = (w + 3) & ~3;
 	break;
     default:
 	xf86DrvMsg( pScrn->scrnIndex, X_ERROR,
-		    "%s: Unsupported bpp %d!\n", __func__, bpp );
+		    "%s: Unsupported cpp %d!\n", __func__, cpp );
 	return NULL;
     }
 
@@ -656,10 +673,10 @@ RADEONHostDataBlit(
      */
     if (info->ChipFamily < CHIP_FAMILY_R300) {
         BEGIN_RING(2);
-	if (bpp == 2)
+	if (cpp == 2)
 	    OUT_RING_REG(RADEON_RBBM_GUICNTL,
 			 RADEON_HOST_DATA_SWAP_HDW);
-	else if (bpp == 1)
+	else if (cpp == 1)
 	    OUT_RING_REG(RADEON_RBBM_GUICNTL,
 			 RADEON_HOST_DATA_SWAP_32BIT);
 	else
@@ -672,16 +689,13 @@ RADEONHostDataBlit(
     /*RADEON_PURGE_CACHE();
       RADEON_WAIT_UNTIL_IDLE();*/
 
-    dst_offs = *dst - info->FB + info->fbLocation;
-    *hpass = min( *h, ( ( RADEON_BUFFER_SIZE - 8 * 4 ) / *bufPitch ) );
+    *hpass = min( *h, ( ( RADEON_BUFFER_SIZE - 10 * 4 ) / *bufPitch ) );
     dwords = *hpass * *bufPitch / 4;
 
-    y = ( dst_offs & 1023 ) / dstPitch;
-    x = ( ( dst_offs & 1023 ) - ( y * dstPitch ) ) / bpp;
-
-    BEGIN_RING( dwords + 8 );
-    OUT_RING( CP_PACKET3( RADEON_CP_PACKET3_CNTL_HOSTDATA_BLT, dwords + 8 - 2 ) );
+    BEGIN_RING( dwords + 10 );
+    OUT_RING( CP_PACKET3( RADEON_CP_PACKET3_CNTL_HOSTDATA_BLT, dwords + 10 - 2 ) );
     OUT_RING( RADEON_GMC_DST_PITCH_OFFSET_CNTL
+	    | RADEON_GMC_DST_CLIPPING
 	    | RADEON_GMC_BRUSH_NONE
 	    | format
 	    | RADEON_GMC_SRC_DATATYPE_COLOR
@@ -689,11 +703,13 @@ RADEONHostDataBlit(
 	    | RADEON_DP_SRC_SOURCE_HOST_DATA
 	    | RADEON_GMC_CLR_CMP_CNTL_DIS
 	    | RADEON_GMC_WR_MSK_DIS );
-    OUT_RING( dstPitch << 16 | dst_offs >> 10 );
+    OUT_RING( dstPitchOff );
+    OUT_RING( (*y << 16) | x );
+    OUT_RING( ((*y + *hpass) << 16) | (x + w) );
     OUT_RING( 0xffffffff );
     OUT_RING( 0xffffffff );
-    OUT_RING( y << 16 | x );
-    OUT_RING( *hpass << 16 | w );
+    OUT_RING( *y << 16 | x );
+    OUT_RING( *hpass << 16 | (*bufPitch / cpp) );
     OUT_RING( dwords );
 
     ret = ( CARD8* )&__head[__count];
@@ -701,7 +717,7 @@ RADEONHostDataBlit(
     __count += dwords;
     ADVANCE_RING();
 
-    *dst += *hpass * dstPitch;
+    *y += *hpass;
     *h -= *hpass;
 
     return ret;
@@ -761,7 +777,7 @@ void RADEONCopySwap(CARD8 *dst, CARD8 *src, unsigned int size, int swap)
 void
 RADEONHostDataBlitCopyPass(
     ScrnInfoPtr pScrn,
-    unsigned int bpp,
+    unsigned int cpp,
     CARD8 *dst,
     CARD8 *src,
     unsigned int hpass,
@@ -769,7 +785,9 @@ RADEONHostDataBlitCopyPass(
     unsigned int srcPitch
 ){
 
+#if X_BYTE_ORDER == X_BIG_ENDIAN
     RADEONInfoPtr info = RADEONPTR( pScrn );
+#endif
 
     /* RADEONHostDataBlitCopy can return NULL ! */
     if( (dst==NULL) || (src==NULL)) return;
@@ -778,7 +796,7 @@ RADEONHostDataBlitCopyPass(
     {
 #if X_BYTE_ORDER == X_BIG_ENDIAN
         if (info->ChipFamily >= CHIP_FAMILY_R300) {
-	    switch(bpp) {
+	    switch(cpp) {
 	    case 1:
 		RADEONCopySwap(dst, src, hpass * dstPitch,
 			       RADEON_HOST_DATA_SWAP_32BIT);
@@ -799,7 +817,7 @@ RADEONHostDataBlitCopyPass(
 	{
 #if X_BYTE_ORDER == X_BIG_ENDIAN
             if (info->ChipFamily >= CHIP_FAMILY_R300) {
-		switch(bpp) {
+		switch(cpp) {
 		case 1:
 		    RADEONCopySwap(dst, src, minPitch,
 				   RADEON_HOST_DATA_SWAP_32BIT);
@@ -812,7 +830,9 @@ RADEONHostDataBlitCopyPass(
 	    }
 #endif
 	    memcpy( dst, src, minPitch );
+#if X_BYTE_ORDER == X_BIG_ENDIAN
 	next:
+#endif
 	    src += srcPitch;
 	    dst += dstPitch;
 	}
